@@ -24,12 +24,13 @@
 //! ```rust
 //! # #[cfg(feature = "tfidf")] {
 //! use jieba_rs::Jieba;
-//! use jieba_rs::{TFIDF, KeywordExtract};
+//! use jieba_rs::{TfIdf, KeywordExtract};
 //!
 //! fn main() {
 //!     let jieba = Jieba::new();
-//!     let keyword_extractor = TFIDF::new_with_jieba(&jieba);
-//!     let top_k = keyword_extractor.extract_tags(
+//!     let keyword_extractor = TfIdf::default();
+//!     let top_k = keyword_extractor.extract_keywords(
+//!         &jieba,
 //!         "今天纽约的天气真好啊，京华大酒店的张尧经理吃了一只北京烤鸭。后天纽约的天气不好，昨天纽约的天气也不好，北京烤鸭真好吃",
 //!         3,
 //!         vec![],
@@ -46,8 +47,9 @@
 //!
 //! fn main() {
 //!     let jieba = Jieba::new();
-//!     let keyword_extractor = TextRank::new_with_jieba(&jieba);
-//!     let top_k = keyword_extractor.extract_tags(
+//!     let keyword_extractor = TextRank::default();
+//!     let top_k = keyword_extractor.extract_keywords(
+//!         &jieba,
 //!         "此外，公司拟对全资子公司吉林欧亚置业有限公司增资4.3亿元，增资后，吉林欧亚置业注册资本由7000万元增加到5亿元。吉林欧亚置业主要经营范围为房地产开发及百货零售等业务。目前在建吉林欧亚城市商业综合体项目。2013年，实现营业收入0万元，实现净利润-139.13万元。",
 //!         6,
 //!         vec![String::from("ns"), String::from("n"), String::from("vn"), String::from("v")],
@@ -72,10 +74,10 @@
 use lazy_static::lazy_static;
 
 use std::cmp::Ordering;
+use std::collections::HashMap;
 use std::io::BufRead;
 
 use cedarwood::Cedar;
-use hashbrown::HashMap;
 use regex::{Match, Matches, Regex};
 
 pub(crate) type FxHashMap<K, V> = HashMap<K, V, fxhash::FxBuildHasher>;
@@ -84,9 +86,13 @@ pub use crate::errors::Error;
 #[cfg(feature = "textrank")]
 pub use crate::keywords::textrank::TextRank;
 #[cfg(feature = "tfidf")]
-pub use crate::keywords::tfidf::TFIDF;
+pub use crate::keywords::tfidf::TfIdf;
 #[cfg(any(feature = "tfidf", feature = "textrank"))]
-pub use crate::keywords::{Keyword, KeywordExtract};
+pub use crate::keywords::{Keyword, KeywordExtract, KeywordExtractConfig, DEFAULT_STOP_WORDS};
+
+#[cfg(any(feature = "tfidf", feature = "textrank"))]
+#[macro_use]
+extern crate derive_builder;
 
 mod errors;
 mod hmm;
@@ -101,7 +107,7 @@ use sparse_dag::StaticSparseDAG;
 
 lazy_static! {
     static ref RE_HAN_DEFAULT: Regex = Regex::new(r"([\u{3400}-\u{4DBF}\u{4E00}-\u{9FFF}\u{F900}-\u{FAFF}\u{20000}-\u{2A6DF}\u{2A700}-\u{2B73F}\u{2B740}-\u{2B81F}\u{2B820}-\u{2CEAF}\u{2CEB0}-\u{2EBEF}\u{2F800}-\u{2FA1F}a-zA-Z0-9+#&\._%\-]+)").unwrap();
-    static ref RE_SKIP_DEAFULT: Regex = Regex::new(r"(\r\n|\s)").unwrap();
+    static ref RE_SKIP_DEFAULT: Regex = Regex::new(r"(\r\n|\s)").unwrap();
     static ref RE_HAN_CUT_ALL: Regex = Regex::new(r"([\u{3400}-\u{4DBF}\u{4E00}-\u{9FFF}\u{F900}-\u{FAFF}\u{20000}-\u{2A6DF}\u{2A700}-\u{2B73F}\u{2B740}-\u{2B81F}\u{2B820}-\u{2CEAF}\u{2CEB0}-\u{2EBEF}\u{2F800}-\u{2FA1F}]+)").unwrap();
     static ref RE_SKIP_CUT_ALL: Regex = Regex::new(r"[^a-zA-Z0-9+#\n]").unwrap();
 }
@@ -203,15 +209,14 @@ pub struct Tag<'a> {
 
 #[derive(Debug, Clone)]
 struct Record {
-    word: String,
     freq: usize,
     tag: String,
 }
 
 impl Record {
     #[inline(always)]
-    fn new(word: String, freq: usize, tag: String) -> Self {
-        Self { word, freq, tag }
+    fn new(freq: usize, tag: String) -> Self {
+        Self { freq, tag }
     }
 }
 
@@ -279,8 +284,7 @@ impl Jieba {
                 self.total -= old_freq;
             }
             None => {
-                self.records
-                    .push(Record::new(String::from(word), freq, String::from(tag)));
+                self.records.push(Record::new(freq, String::from(tag)));
                 let word_id = (self.records.len() - 1) as i32;
 
                 self.cedar.update(word, word_id);
@@ -306,7 +310,7 @@ impl Jieba {
         while dict.read_line(&mut buf)? > 0 {
             {
                 line_no += 1;
-                let mut iter = buf.trim().split_whitespace();
+                let mut iter = buf.split_whitespace();
                 if let Some(word) = iter.next() {
                     let freq = iter
                         .next()
@@ -331,8 +335,7 @@ impl Jieba {
                             self.records[word_id as usize].freq = freq;
                         }
                         None => {
-                            self.records
-                                .push(Record::new(String::from(word), freq, String::from(tag)));
+                            self.records.push(Record::new(freq, String::from(tag)));
                             let word_id = (self.records.len() - 1) as i32;
                             self.cedar.update(word, word_id);
                         }
@@ -353,7 +356,7 @@ impl Jieba {
         }
     }
 
-    /// Suggest word frequency to force the characters in a word to be joined or splitted.
+    /// Suggest word frequency to force the characters in a word to be joined or split.
     pub fn suggest_freq(&self, segment: &str) -> usize {
         let logtotal = (self.total as f64).ln();
         let logfreq = self.cut(segment, false).iter().fold(0f64, |freq, word| {
@@ -495,9 +498,7 @@ impl Jieba {
         words: &mut Vec<&'a str>,
         route: &mut Vec<(f64, usize)>,
         dag: &mut StaticSparseDAG,
-        V: &mut Vec<f64>,
-        prev: &mut Vec<Option<hmm::Status>>,
-        path: &mut Vec<hmm::Status>,
+        hmm_context: &mut hmm::HmmContext,
     ) {
         self.dag(sentence, dag);
         self.calc(sentence, dag, route);
@@ -523,7 +524,7 @@ impl Jieba {
                     if word.chars().count() == 1 {
                         words.push(word);
                     } else if self.cedar.exact_match_search(word).is_none() {
-                        hmm::cut_with_allocated_memory(word, words, V, prev, path);
+                        hmm::cut_with_allocated_memory(word, words, hmm_context);
                     } else {
                         let mut word_indices = word.char_indices().map(|x| x.0).peekable();
                         while let Some(byte_start) = word_indices.next() {
@@ -573,17 +574,13 @@ impl Jieba {
     fn cut_internal<'a>(&self, sentence: &'a str, cut_all: bool, hmm: bool) -> Vec<&'a str> {
         let heuristic_capacity = sentence.len() / 2;
         let mut words = Vec::with_capacity(heuristic_capacity);
-        let re_han: &Regex = if cut_all { &*RE_HAN_CUT_ALL } else { &*RE_HAN_DEFAULT };
-        let re_skip: &Regex = if cut_all { &*RE_SKIP_CUT_ALL } else { &*RE_SKIP_DEAFULT };
-        let splitter = SplitMatches::new(&re_han, sentence);
+        let re_han: &Regex = if cut_all { &RE_HAN_CUT_ALL } else { &RE_HAN_DEFAULT };
+        let re_skip: &Regex = if cut_all { &RE_SKIP_CUT_ALL } else { &RE_SKIP_DEFAULT };
+        let splitter = SplitMatches::new(re_han, sentence);
         let mut route = Vec::with_capacity(heuristic_capacity);
         let mut dag = StaticSparseDAG::with_size_hint(heuristic_capacity);
 
-        let R = 4;
-        let C = sentence.chars().count();
-        let mut V = if hmm { vec![0.0; R * C] } else { Vec::new() };
-        let mut prev: Vec<Option<hmm::Status>> = if hmm { vec![None; R * C] } else { Vec::new() };
-        let mut path: Vec<hmm::Status> = if hmm { vec![hmm::Status::B; C] } else { Vec::new() };
+        let mut hmm_context = hmm::HmmContext::new(sentence.chars().count());
 
         for state in splitter {
             match state {
@@ -594,7 +591,7 @@ impl Jieba {
                     if cut_all {
                         self.cut_all_internal(block, &mut words);
                     } else if hmm {
-                        self.cut_dag_hmm(block, &mut words, &mut route, &mut dag, &mut V, &mut prev, &mut path);
+                        self.cut_dag_hmm(block, &mut words, &mut route, &mut dag, &mut hmm_context);
                     } else {
                         self.cut_dag_no_hmm(block, &mut words, &mut route, &mut dag);
                     }
@@ -603,7 +600,7 @@ impl Jieba {
                     let block = state.into_str();
                     assert!(!block.is_empty());
 
-                    let skip_splitter = SplitMatches::new(&re_skip, block);
+                    let skip_splitter = SplitMatches::new(re_skip, block);
                     for skip_state in skip_splitter {
                         let word = skip_state.into_str();
                         if word.is_empty() {
@@ -820,18 +817,18 @@ mod tests {
     fn test_split_matches() {
         let re_han = &*RE_HAN_DEFAULT;
         let splitter = SplitMatches::new(
-            &re_han,
+            re_han,
             "👪 PS: 我觉得开源有一个好处，就是能够敦促自己不断改进 👪，避免敞帚自珍",
         );
         for state in splitter {
             match state {
                 SplitState::Matched(_) => {
                     let block = state.into_str();
-                    assert_eq!(block.is_empty(), false);
+                    assert!(!block.is_empty());
                 }
                 SplitState::Unmatched(_) => {
                     let block = state.into_str();
-                    assert_eq!(block.is_empty(), false);
+                    assert!(!block.is_empty());
                 }
             }
         }
@@ -840,7 +837,7 @@ mod tests {
     #[test]
     fn test_split_matches_against_unicode_sip() {
         let re_han = &*RE_HAN_DEFAULT;
-        let splitter = SplitMatches::new(&re_han, "讥䶯䶰䶱䶲䶳䶴䶵𦡦");
+        let splitter = SplitMatches::new(re_han, "讥䶯䶰䶱䶲䶳䶴䶵𦡦");
 
         let result: Vec<&str> = splitter.map(|x| x.into_str()).collect();
         assert_eq!(result, vec!["讥䶯䶰䶱䶲䶳䶴䶵𦡦"]);
